// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit

/// Defines formatting options for strings
public enum StringFormatOption {
    /// Line height
    case lineHeight(CGFloat)
    
    /// Line spacing
    case lineSpacing(CGFloat)
    
    /// Font
    case font(UIFont)
    
    /// Text color
    case textColor(UIColor)
    
    /// Letter spacing aka. kern
    case letterSpacing(CGFloat)
    
    /// Line break mode (e.g. truncating)
    case lineBreakMode(NSLineBreakMode)
    
    /// Text alignment for the paragraph
    case textAlignment(NSTextAlignment)
}

public extension String {
    /// Returns the number of characters in the string.
    var length: Int {
        return self.count
    }

    /// Will return nil if string is empty.
    var nilIfEmpty: String? {
        if trimmingCharacters(in: .whitespacesAndNewlines).isEmpty {
            return nil
        }

        return self
    }

    /// Returns a url encoded strings with alphanumerics as allowed characters
    var urlEncoded: String {
        let allowedCharacterSet: CharacterSet = CharacterSet(charactersIn: "!*'();:@&=+$,/?%#[] ").inverted
        return self.addingPercentEncoding(withAllowedCharacters: allowedCharacterSet) ?? self
    }
    

    /// Converts the receiver into a `NSMutableAttributedString` based on the provided options.
    ///
    /// Note that you can build project-specific option-sets by implementing a `StringFormatOptions` class with static functions returning sets of options.
    func convertToAttributed(withOptions options: [StringFormatOption]) -> NSMutableAttributedString {
        let range: NSRange = NSRange(location: 0, length: self.count)
        return self.format(withOptions: options, inRange: range)
    }
    
    /// Converts the receiver into a `NSMutableAttributedString` based on the provided options within a specified range.
    ///
    /// Note that you can build project-specific option-sets by implementing a `StringFormatOptions` class with static functions returning sets of options.
    func format(withOptions options: [StringFormatOption], inRange range: NSRange) -> NSMutableAttributedString {
        let attributedString: NSMutableAttributedString = NSMutableAttributedString(string: self)
        let paragraphStyle: NSMutableParagraphStyle = NSMutableParagraphStyle()
        paragraphStyle.lineBreakMode = .byTruncatingTail //default to truncate tail
        options.forEach { (option: StringFormatOption) in
            self.format(forOption: option, forAttributedString: attributedString, withParagraphStyle: paragraphStyle, range: range)
        }
        attributedString.addAttribute(.paragraphStyle, value: paragraphStyle, range: range)
        return attributedString
    }
    
    /// Formats one option for an attributed string with a `paragraphStyle` in a specified range
    ///
    /// The range is not used if you pass an option, which is set on the `paragraphStyle`. It is your responsibility to add the `paragraphStyle` to the right range.
    func format(forOption option: StringFormatOption, forAttributedString attributedString: NSMutableAttributedString, withParagraphStyle paragraphStyle: NSMutableParagraphStyle, range: NSRange) {
        switch option {
        case .textColor(let color):
            attributedString.addAttribute(.foregroundColor, value: color, range: range)
        case .font(let font):
            attributedString.addAttribute(.font, value: font, range: range)
        case .lineHeight(let lineHeight):
            paragraphStyle.minimumLineHeight = lineHeight
            paragraphStyle.maximumLineHeight = lineHeight
        case .lineSpacing(let lineSpacing):
            paragraphStyle.lineSpacing = lineSpacing
        case .letterSpacing(let letterSpacing):
            attributedString.addAttribute(.kern, value: letterSpacing, range: range)
        case .lineBreakMode(let lineBreakMode):
            paragraphStyle.lineBreakMode = lineBreakMode
        case .textAlignment(let textAlignment):
            paragraphStyle.alignment = textAlignment
        }
    }
}
