// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import XCTest
@testable import AppleExtensions

class StringExtensionTests: XCTestCase {

    func testLength() {
        let string: String = "Another test string 🎈"
        XCTAssertEqual(string.length, 21)
    }

    func testConvertStringToAttributed() {
        let font = UIFont.systemFont(ofSize: 12)
        let color = UIColor.magenta

        let letterSpacing: CGFloat = 5.0
        let lineSpacing: CGFloat = 3.0
        let lineHeight: CGFloat = 10.0

        let textAlignment: NSTextAlignment = .center
        let lineBreakMode: NSLineBreakMode = .byWordWrapping

        let string = "This is my awesome string"
        let attributedString: NSMutableAttributedString = string.convertToAttributed(withOptions: [
            .font(font),
            .textColor(color),
            .letterSpacing(letterSpacing),
            .lineSpacing(lineSpacing),
            .lineHeight(lineHeight),
            .textAlignment(textAlignment),
            .lineBreakMode(lineBreakMode)
        ])

        let attributes: [NSAttributedString.Key: Any] = attributedString.attributes(at: 0, effectiveRange: nil)
        XCTAssert((attributes[NSAttributedString.Key.paragraphStyle] as? NSParagraphStyle)?.lineSpacing == lineSpacing, "The line spacing should be \(lineSpacing)")
        XCTAssert((attributes[NSAttributedString.Key.paragraphStyle] as? NSParagraphStyle)?.minimumLineHeight == lineHeight, "The line height should be \(lineHeight)")
        XCTAssert((attributes[NSAttributedString.Key.paragraphStyle] as? NSParagraphStyle)?.maximumLineHeight == lineHeight, "The line height should be \(lineHeight)")
        XCTAssert((attributes[NSAttributedString.Key.paragraphStyle] as? NSParagraphStyle)?.alignment == textAlignment, "The text alignment should be \(textAlignment)")
        XCTAssert((attributes[NSAttributedString.Key.paragraphStyle] as? NSParagraphStyle)?.lineBreakMode == lineBreakMode, "The line break mode should be \(lineBreakMode)")

        XCTAssert((attributes[NSAttributedString.Key.font] as? UIFont)?.fontName == font.fontName, "The font name should be \(font.fontName)")
        XCTAssert((attributes[NSAttributedString.Key.font] as? UIFont)?.pointSize == font.pointSize, "The size should be \(String(describing: font.pointSize))")
        XCTAssert((attributes[NSAttributedString.Key.foregroundColor] as? UIColor) == color, "The color should be \(color.description)")
        XCTAssert((attributes[NSAttributedString.Key.kern] as? CGFloat) == letterSpacing, "The letter spacing should be \(String(describing: letterSpacing))")
    }

    func testUrlEncoded() {
        let string: String = "http://my-important-url.com?withCoolParameters=coolParamters&andOtherStuffWithSpaces=some stuff"
        XCTAssertEqual("http%3A%2F%2Fmy-important-url.com%3FwithCoolParameters%3DcoolParamters%26andOtherStuffWithSpaces%3Dsome%20stuff", string.urlEncoded)
    }

    func testNilIfEmpty() {
        let emptyString = ""
        let whiteSpaceString = " "
        let nonNilString = " 1 "

        XCTAssertNil(emptyString.nilIfEmpty)
        XCTAssertNil(whiteSpaceString.nilIfEmpty)
        XCTAssertEqual(nonNilString, nonNilString.nilIfEmpty)
    }
}
