// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

extension View {

    @ViewBuilder
    func ifLet<V, Transform: View>(
        _ value: V?,
        transform: (Self, V) -> Transform
    ) -> _ConditionalContent<Transform, Self> {
        if let value = value {
            transform(self, value)
        } else {
            self
        }
    }

    @ViewBuilder
    func alert<E : Error>(for error: Binding<E?>) -> some View {
        self.alert(
            isPresented: Binding(
                    get: { error.wrappedValue != nil },
                    set: { isPresented in
                        if !isPresented && error.wrappedValue != nil {
                            error.wrappedValue = nil
                        }
                    }),
            content: {
                Alert.error(error.wrappedValue)
        })
    }

    @ViewBuilder
    func applyDefaultShadow() -> some View {
        shadow(color: R.color.shadowSolid.color().opacity(0.3), radius: 8, x: 0, y: 2)
    }

    @ViewBuilder
    func applyDefaultCardShadow(yOffset: CGFloat = 0) -> some View {
        shadow(color: R.color.cardShadow.color().opacity(0.7), radius: 5, x: 0, y: yOffset)
    }

    /// Positions this view within an invisible frame with the specified size.
    /// - Parameters:
    ///   - size: A fixed size for the resulting view.
    ///   - alignment: The alignment of this view inside the resulting view. alignment applies if this view is smaller than the size given by the resulting frame.
    /// - Returns: A view with fixed dimensions of size.
    @ViewBuilder
    func frame(size: CGFloat, alignment: Alignment = .center) -> some View {
        frame(width: size, height: size, alignment: alignment)
    }
}
