// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import ErrorData
import MaternityData

protocol UserContextProtocol {

    /// Indicates whether the user has handled the onboarding flow (either logged in or skipped it)
    var hasHandledOnboardingFlow: Bool { get }

    /// Returns true if the user is authenticated, if false the user did not login with NemID on onboarding
    var isAuthenticated: Bool { get }

    /// Returns `true` if TIM has any user registered in the storage, otherwise `false`
    var hasUserRegistered: Bool { get }

    /// `nil` if the user did not login with NemID
    var userId: String? { get }

    /// The user can switch between careplans
    var selectedCarePlan: String? { get }

    /// The user data, i.e. name, CPR, maternity dates, etc.
    var userDataCard: DataCard? { get }

    /// Logs out and shows the log in screen
    func logout()

    /// Logs out, and removes all data stored for the user.
    /// This will present the onboarding flow immediately after the cleanup
    func resetAllUserData()

    /// Sets the `hasHandledOnboardingFlow` property.
    /// The value is persisted.
    func setHasHandledOnboardingFlow(_ handled: Bool)

    /// Fetches all initial data for the user, which the system cannot function without.
    /// Should always be called upon login and the user should never be able to continue without a successful completion on this.
    func fetchAllInitialData() -> AnyPublisher<Void, ServiceError>
}
