// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

// swiftlint:disable file_length
import Foundation
import MaternityServices
import MaternityData
import CMSServices
import Combine
import ErrorData
import CMSData

// swiftlint:disable:next type_body_length
final class AroundThePregnancyServiceImpl: AroundThePregnancyService {
    @DI private var userContext: UserContextProtocol
    @DI private var dictionaryService: DictionaryService

    func fetchExpectedDateOfBirth() -> AnyPublisher<[KeyValueModel], ServiceError> {
        guard let expectedDateOfDeliverySection = userContext.userDataCard?.expectedDateOfDeliverySection else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }

        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_expected_date_of_birth_last_first_day_of_menstrual_cycle(),
                        primaryValue: expectedDateOfDeliverySection.expectedDateOfDeliveryFromLastPeriod?.lastFirstDayOfCycle,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_expected_date_of_birth_days_in_cycle(),
                        primaryValue: expectedDateOfDeliverySection.expectedDateOfDeliveryFromLastPeriod?.menstrualCycle?.inDays?.description,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_expected_date_of_birth_calculated_date_of_birth(),
                        primaryValue: expectedDateOfDeliverySection.expectedDateOfDeliveryFromLastPeriod?.calculatedExpectedDeliveryDate,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_expected_date_of_birth_cycle_certainty(),
                        primaryValue: expectedDateOfDeliverySection.expectedDateOfDeliveryFromLastPeriod?.certainty?.description,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_expected_date_of_birth_remarks(),
                        primaryValue: expectedDateOfDeliverySection.remarks
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    func fetchAssistedFertility() -> AnyPublisher<[KeyValueModel], ServiceError> {
        guard let assistedFertilizationSection = userContext.userDataCard?.assistedFertilizationSection else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }

        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_assisted_fertilization_type(),
                        primaryValue: assistedFertilizationSection.type?.description,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_assisted_fertilization_remarks(),
                        primaryValue: assistedFertilizationSection.remarks
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    func fetchRiskAssessment() -> AnyPublisher<[KeyValueModel], ServiceError> {
        guard let preliminaryRiskAssessment = userContext.userDataCard?.prenatalRiskAssessmentSection?.preliminaryRiskAssessment else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }

        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_risk_assessment_family_history(),
                        primaryValue: preliminaryRiskAssessment.familyHistory,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_risk_assessment_consanguinity(),
                        primaryValue: preliminaryRiskAssessment.consanguinity?.localized ,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_risk_assessment_requested_scan_first_trimester(),
                        primaryValue: preliminaryRiskAssessment.firstTrimesterUltrasoundScanOfFetusRequested?.localized,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_risk_assessment_requested_scan_second_trimester(),
                        primaryValue: preliminaryRiskAssessment.secondTrimesterUltrasoundScanOfFetusRequested?.localized,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_risk_assessment_wants_risk_assessment(),
                        primaryValue: preliminaryRiskAssessment.prenatalRiskAssessmentRequested?.localized,
                        dictionary: dictionary
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    func fetchWorkEnvironment() -> AnyPublisher<[KeyValueModel], ServiceError> {
        guard let workEnvironmentImpact = userContext.userDataCard?.workEnvironmentImpactSection else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }

        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_work_environment_mothers_job(),
                        primaryValue: workEnvironmentImpact.mother?.jobType,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_work_environment_mothers_work_hours_per_week(),
                        primaryValue: workEnvironmentImpact.mother?.hoursOfWorkPerWeek?.description ,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_work_environment_exposure_to_risk(),
                        primaryValue: workEnvironmentImpact.mother?.exposureToWorkEnvironmentRisk?.types?.map { $0.description }.joined(separator: ", "),
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_work_environment_type_of_exposure(),
                        primaryValue: workEnvironmentImpact.mother?.exposureToWorkEnvironmentRisk?.remarks,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_work_environment_duration_of_exposure(),
                        primaryValue: workEnvironmentImpact.mother?.exposureToWorkEnvironmentRisk?.exposureDuration,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_work_environment_referred_to_occupational_health_department(),
                        primaryValue: workEnvironmentImpact.mother?.referredToOccupationalHealthDepartmentForRiskAssessment?.localized,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_work_environment_sick_leave(),
                        primaryValue: workEnvironmentImpact.mother?.prenatalSickLeave?.description,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_work_environment_partners_job(),
                        primaryValue: workEnvironmentImpact.partner?.jobType,
                        dictionary: dictionary
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    func fetchHealth() -> AnyPublisher<[KeyValueModelGroup], ServiceError> {
        guard let userDataCard = userContext.userDataCard else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }

        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map({ (dictionary: CMSDictionary) -> [KeyValueModelGroup] in
                [
                    userDataCard.allergiesSection.map { self.mapAllergyGroup(allergiesSection: $0, dictionary: dictionary) },
                    userDataCard.chronicalDiseasesSection.map { self.mapChronicConditionsGroup(chronicDiseasesSection: $0, dictionary: dictionary) },
                    userDataCard.medicineSection.map { self.mapMedicineGroup(medicineSection: $0, dictionary: dictionary) },
                    userDataCard.mmrVacinationSection.map { self.mapVaccinationGroup(vaccinationSection: $0, dictionary: dictionary) },
                    self.maphistoryOfPreviousTreatmentWithImpactOnPregnancySection(userDataCard: userDataCard, dictionary: dictionary)
                ]
                .compactMap { $0 }
            })
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    func fetchLifestyle() -> AnyPublisher<[KeyValueModelGroup], ServiceError> {
        guard let userDataCard = userContext.userDataCard else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }

        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModelGroup] in
                [
                    userDataCard.tobaccoUseAndExposureSection.map { self.mapTobaccoUseAndExposureSectionGroup(tobaccoUseAndExposureSection: $0, dictionary: dictionary) },
                    userDataCard.alcoholDrinkingBehaviorSection.map { self.mapAlcoholGroup(alcoholDrinkingBehaviorSection: $0, dictionary: dictionary) },
                    userDataCard.substanceUseSection.map { self.mapSubstanceUseGroup(substanceUseSection: $0, dictionary: dictionary) },
                    userDataCard.dietaryAndExerciseSection.map { self.mapDietaryAndExerciseGroup(dietaryAndExerciseSection: $0, dictionary: dictionary) }
                ]
                .compactMap { $0 }
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    // MARK: - Health helpers
    private func mapAllergyGroup(allergiesSection: AllergiesSection, dictionary: CMSDictionary) -> KeyValueModelGroup {
        KeyValueModelGroup(title: R.string.localizable.journal_section_health_allergies_title(),
                           keyValueModels: [
                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_health_maternal_allergies(),
                                primaryValue: allergiesSection.allergiesOfMother,
                                dictionary: dictionary
                            ),
                            
                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_health_child_predisposition(),
                                primaryValue: allergiesSection.allergicPredispositionOfChild,
                                dictionary: dictionary
                            )
                           ])
    }

    private func mapChronicConditionsGroup(chronicDiseasesSection: ChronicDiseasesSection, dictionary: CMSDictionary) -> KeyValueModelGroup {
        KeyValueModelGroup(title: R.string.localizable.journal_section_health_chronic_diseases_title(),
                           keyValueModels: [
                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_health_chronic_diseases_relevant_to_pregnancy(),
                                primaryValue: chronicDiseasesSection.conditions?.map { $0.description }.joined(separator: ", "),
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_health_chronic_diseases_remarks(),
                                primaryValue: chronicDiseasesSection.comment,
                                dictionary: dictionary
                            )
                           ])
    }

    private func mapMedicineGroup(medicineSection: MedicineSection, dictionary: CMSDictionary) -> KeyValueModelGroup {
        KeyValueModelGroup(title: R.string.localizable.journal_section_health_medicine_title(),
                           keyValueModels: [
                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_health_information_about_medicine(),
                                primaryValue: medicineSection.realMedicine,
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_health_information_about_nature_medicine(),
                                primaryValue: medicineSection.natureMedicine,
                                dictionary: dictionary
                            )
                           ])
    }

    private func mapVaccinationGroup(vaccinationSection: VaccinationSection, dictionary: CMSDictionary) -> KeyValueModelGroup {
        KeyValueModelGroup(title: R.string.localizable.journal_section_health_medicine_title(),
                           keyValueModels: [
                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_health_mfr_vaccination(),
                                primaryValue: vaccinationSection.status?.description,
                                dictionary: dictionary
                            )
                           ])
    }

    private func maphistoryOfPreviousTreatmentWithImpactOnPregnancySection(userDataCard: DataCard, dictionary: CMSDictionary) -> KeyValueModelGroup {
        KeyValueModelGroup(title: R.string.localizable.journal_section_health_previous_treatment_with_impact_on_pregnancy_title(),
                           keyValueModels: [
                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_health_previous_treatment_with_impact_on_pregnancy(),
                                primaryValue: userDataCard.historyOfPreviousTreatmentWithImpactOnPregnancySection,
                                dictionary: dictionary
                            )
                           ])
    }

    // MARK: - Lifestyle Helpers
    private func mapTobaccoUseAndExposureSectionGroup(tobaccoUseAndExposureSection: TobaccoUseAndExposureSection, dictionary: CMSDictionary) -> KeyValueModelGroup {
        KeyValueModelGroup(title: R.string.localizable.journal_section_lifestyle_tobacco_title(),
                           keyValueModels: [
                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_tobacco_before_pregnancy(),
                                primaryValue: tobaccoUseAndExposureSection.tobaccoUser?.description,
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_tobacco_end_date(),
                                primaryValue: tobaccoUseAndExposureSection.tobaccoUser?.description,
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_tobacco_smoking_during_pregnancy(),
                                primaryValue: tobaccoUseAndExposureSection.smokeDuringPregnancy?.description,
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_tobacco_exposure_to_environmental_smoke(),
                                primaryValue: tobaccoUseAndExposureSection.exposureToEnvironmentalTobaccoSmoke?.description,
                                dictionary: dictionary
                            )
                           ])
    }

    private func mapAlcoholGroup(alcoholDrinkingBehaviorSection: AlcoholDrinkingBehaviorSection, dictionary: CMSDictionary) -> KeyValueModelGroup {
        KeyValueModelGroup(title: R.string.localizable.journal_section_lifestyle_alcohol_title(),
                           keyValueModels: [
                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_alcohol_before_usage(),
                                primaryValue: alcoholDrinkingBehaviorSection.weeklyConsumptionBeforePregnancy?.consumesAlcohol.description,
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_alcohol_before_average_consumption(),
                                primaryValue: alcoholDrinkingBehaviorSection.weeklyConsumptionBeforePregnancy?.consumption,
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_alcohol_during_usage(),
                                primaryValue: alcoholDrinkingBehaviorSection.weeklyConsumptionDuringPregnancy?.consumesAlcohol.description,
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_alcohol_during_average_consumption(),
                                primaryValue: alcoholDrinkingBehaviorSection.weeklyConsumptionDuringPregnancy?.consumption,
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_alcohol_during_excessive_drinking(),
                                primaryValue: alcoholDrinkingBehaviorSection.weeklyConsumptionDuringPregnancy?.excessiveDrinkingCount?.description,
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_alcohol_during_excessive_drinking_weeks(),
                                primaryValue: alcoholDrinkingBehaviorSection.weeklyConsumptionDuringPregnancy?.excessiveDrinkingWeeks?.map { $0.description }.joined(separator: ", "),
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_alcohol_tweak_points(),
                                primaryValue: alcoholDrinkingBehaviorSection.weeklyConsumptionDuringPregnancy?.tweakPoints,
                                dictionary: dictionary
                            )
                           ])
    }

    private func mapSubstanceUseGroup(substanceUseSection: SubstanceUseSection, dictionary: CMSDictionary) -> KeyValueModelGroup {
        KeyValueModelGroup(title: R.string.localizable.journal_section_lifestyle_substance_use_title(),
                           keyValueModels: [
                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_substance_use_before_pregnancy(),
                                primaryValue: substanceUseSection.substanceUseBeforePregnancy?.substanceUse.description,
                                dictionary: dictionary),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_substance_use_before_pregnancy_text(),
                                primaryValue: substanceUseSection.substanceUseBeforePregnancy?.text),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_substance_use_during_pregnancy(),
                                primaryValue: substanceUseSection.substanceUseDuringPregnancy?.substanceUse.description,
                                dictionary: dictionary
                            ),

                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_substance_use_during_pregnancy_text(),
                                primaryValue: substanceUseSection.substanceUseDuringPregnancy?.text
                            )
                           ])
    }

    private func mapDietaryAndExerciseGroup(dietaryAndExerciseSection: DietaryAndExerciseSection, dictionary: CMSDictionary) -> KeyValueModelGroup {
        KeyValueModelGroup(title: R.string.localizable.journal_section_lifestyle_dietary_and_exercise_title(),
                           keyValueModels: [
                            KeyValueModel.dictionaryModel(
                                title: R.string.localizable.journal_section_lifestyle_dietary_and_exercise_text(),
                                primaryValue: dietaryAndExerciseSection.text,
                                dictionary: dictionary
                            )
                           ])
    }
}
// swiftlint:enable file_length
