// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import MaternityData
import MaternityServices
import Combine
import ErrorData
import CMSServices

final class PersonaDataServiceImpl: PersonaDataService {
    @DI private var userContext: UserContextProtocol
    @DI private var dictionaryService: DictionaryService
    
    func fetchContactInformation() -> AnyPublisher<[KeyValueModel], ServiceError> {
        guard let personalData = userContext.userDataCard?.personalDataSection else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }
        
        let data = [
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_personal_data_personal_cpr(),
                primaryValue: personalData.cpr
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_personal_data_personal_name(),
                primaryValue: personalData.name?.fullName
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_personal_data_personal_address(),
                primaryValue: personalData.address?.streetAndNumber),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_personal_data_personal_zip(),
                primaryValue: personalData.address?.zipAndCity
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_personal_data_personal_phonenumber_mobile(),
                primaryValue: personalData.telecoms?.mobile
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_personal_data_personal_phonenumber_home(),
                primaryValue: personalData.telecoms?.home
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_personal_data_personal_phonenumber_work(),
                primaryValue: personalData.telecoms?.work
            ),
            
            personalData.temporaryAddress?.fullAddressAsSingleLine != nil ?
                KeyValueModel.dictionaryModel(
                    title: R.string.localizable.journal_section_personal_data_personal_temporary_adress(),
                    primaryValue: personalData.temporaryAddress?.fullAddressAsSingleLine
                ) : nil,
            
            personalData.temporaryAddress?.duration != nil ?
                KeyValueModel.dictionaryModel(
                    title: R.string.localizable.journal_section_personal_data_personal_temporary_adress_duration(),
                    primaryValue: personalData.temporaryAddress?.duration
                ) : nil
            
        ]
        .compactMap { $0 }
        
        return Just(data)
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    // swiftlint:disable:next function_body_length - This is hardcode values, nothing dangerous here.
    func fetchPracticalInformation() -> AnyPublisher<[KeyValueModel], ServiceError> {
        guard let socialHistory = userContext.userDataCard?.socialHistorySection,
              let languageSection = userContext.userDataCard?.languageSection else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }
        
        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_practical_partnership_status(),
                        primaryValue: socialHistory.maritalOrPartnershipStatus?.description,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_practical_living_conditions(),
                        primaryValue: socialHistory.livingConditions?.description,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_practical_nationality(),
                        primaryValue: languageSection.nationality,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_practical_need_translator(),
                        primaryValue: languageSection.translationRequired.description,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_practical_translator_language(),
                        primaryValue: languageSection.language,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_practical_partners_name(),
                        primaryValue: socialHistory.partner?.name,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_practical_partners_cpr(),
                        primaryValue: socialHistory.partner?.cpr,
                        dictionary: dictionary
                    ),
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_practical_additional_info(),
                        primaryValue: socialHistory.additionalInformations,
                        dictionary: dictionary
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    func fetchNextOfKin() -> AnyPublisher<[KeyValueModelGroup], ServiceError> {
        guard let nextOfKin = userContext.userDataCard?.personalDataSection?.nextOfKin else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }
        
        let data = nextOfKin.enumerated().map { enumerator -> KeyValueModelGroup in
            KeyValueModelGroup(
                title: R.string.localizable.journal_section_personal_data_next_of_kin_title(
                    nextOfKin.count == 1 ? "" : "\(enumerator.offset + 1)"
                ),
                keyValueModels: [
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_next_of_kin_name(),
                        primaryValue: enumerator.element.name
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_next_of_kin_relation(),
                        primaryValue: enumerator.element.relation
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_next_of_kin_phonenumber_mobile(),
                        primaryValue: enumerator.element.telecoms?.mobile
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_next_of_kin_phonenumber_home(),
                        primaryValue: enumerator.element.telecoms?.home
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_next_of_kin_phonenumber_work(),
                        primaryValue: enumerator.element.telecoms?.work
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_personal_data_next_of_kin_additional_info(),
                        primaryValue: enumerator.element.additionalInformation
                    )
                ])
        }
        
        return Just(data)
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
}
