// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import MaternityServices
import MaternityData
import CMSServices
import Combine
import ErrorData
import CMSData

final class PreviousPregnanciesServiceImpl: PreviousPregnanciesService {
    @DI private var userContext: UserContextProtocol
    @DI private var dictionaryService: DictionaryService
    
    func fetchPreviousPregnancies() -> AnyPublisher<PreviousPregnancies, ServiceError> {
        guard let dataCard = userContext.userDataCard else {
            return Just(PreviousPregnancies(abortions: [], births: []))
                .mapToServiceError()
                .eraseToAnyPublisher()
        }
        
        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> PreviousPregnancies in
                let births = dataCard.previousBirthsSection?.births.map { self.mapBirth($0, dictionary: dictionary) } ?? []
                let abortions = dataCard.previousAbortionsSection?.abortions?.map(self.mapAbortion) ?? []
                
                return PreviousPregnancies(abortions: abortions, births: births)
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    private func mapBirth(_ birth: PreviousBirthsSectionBirth, dictionary: CMSDictionary) -> Birth {
        Birth(table: birth.children?.map { mapChild($0, birth: birth, dictionary: dictionary) } ?? [],
              info: [
                KeyValueModel.dictionaryModel(
                    title: R.string.localizable.journal_section_previous_births_child_year(),
                    primaryValue: birth.date?.year.description
                ),
                KeyValueModel.dictionaryModel(
                    title: R.string.localizable.journal_section_previous_births_child_gestation_length(),
                    primaryValue: birth.gestationLength?.description,
                    dictionary: dictionary
                ),
                KeyValueModel.dictionaryModel(
                    title: R.string.localizable.journal_section_previous_births_child_place_of_birth(),
                    primaryValue: birth.birthPlace
                ),
                KeyValueModel.dictionaryModel(
                    title: R.string.localizable.journal_section_previous_births_pregnancy_summary(),
                    primaryValue: birth.pregnancySummary
                ),
                KeyValueModel.dictionaryModel(
                    title: R.string.localizable.journal_section_previous_births_birth_summary(),
                    primaryValue: birth.birthSummary
                ),
                KeyValueModel.dictionaryModel(
                    title: R.string.localizable.journal_section_previous_births_child_subjective_experience(),
                    primaryValue: birth.subjectiveBirthExpirience?.description
                )
              ])
    }
    
    private func mapChild(_ child: PreviousBirthsSectionBirthChild, birth: PreviousBirthsSectionBirth, dictionary: CMSDictionary) -> [KeyValueModel] {
        [
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_births_child_year(),
                primaryValue: birth.date?.year.description
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_births_child_sex(),
                primaryValue: child.sex?.description
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_births_child_weight(),
                primaryValue: child.birthWeight?.description
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_births_child_outcome(),
                primaryValue: child.outcome?.description
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_births_child_current_health(),
                primaryValue: child.childsCurrentHealthStatus
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_births_child_gestation_length(),
                primaryValue: birth.gestationLength?.description,
                dictionary: dictionary
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_births_child_place_of_birth(),
                primaryValue: birth.birthPlace
            )
        ]
    }
    
    private func mapAbortion(_ abortion: PreviousAbortionsSectionAbortion) -> Abortion {
        Abortion(table: [
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_abortions_abortion_year(),
                primaryValue: abortion.date?.year.description
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_abortions_abortion_gestation_length(),
                primaryValue: abortion.gestationLength?.description
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_abortions_abortion_type(),
                primaryValue: abortion.type?.description
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_previous_abortions_abortion_indication(),
                primaryValue: abortion.indication
            )
        ])
    }
}
