// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import MaternityData
import MapKit

struct AppointmentDetailView: View, ScreenTrackable {
    @ObservedObject var viewModel: ViewModel

    init(activity: Activity) {
        viewModel = ViewModel(activity: activity)
    }

    var body: some View {
        ScrollView {
            VStack(alignment: .leading, spacing: 0) {
                if let location = viewModel.activity.location, viewModel.activity.status == .planned {
                    AppointmentDetailsMapView(
                        location: location,
                        directionsButtonWasActivated: $viewModel.isDirectionsActionSheetPresented
                    )
                    .frame(height: 288)
                }
                VStack(alignment: .leading) {
                    AppointmentDetailTitleHeader(
                        title: viewModel.activity.title,
                        subtitle: viewModel.activity.kind.capitalized,
                        badgeText: viewModel.badgeText
                    )
                    .padding(.bottom, 16)
                    if let plannedTime = viewModel.activity.plannedTime,
                       let location = viewModel.activity.location, viewModel.isPlannedOrFinished {
                        AppointmentDetailPlannedValues(
                            from: plannedTime.from,
                            to: plannedTime.to,
                            location: location.address.formatted,
                            practitioner: viewModel.activity.practitioner.fullName,
                            phoneNumber: location.phone,
                            email: location.email
                        )
                    } else {
                        AppointmentDetailExpectedValues(
                            expectedDate: R.string.localizable.appointment_details_estimated_date_value(viewModel.activity.gestationWeek)
                        )
                    }
                }
                .padding()
                .background(R.color.viewBackground.color())
                BottomContainerCardView {
                    AppointmentDetailContentPickerView(
                        activity: viewModel.activity,
                        dictionary: viewModel.dictionary
                    )
                    .padding([.horizontal, .bottom])
                }
            }
        }
        .navigationBarTitle(Text(R.string.localizable.appointment_details_title()), displayMode: .inline)
        .background(R.color.cardBackground.color())
        .onAppear(perform: trackScreenAppearing)
        .onAppear(perform: viewModel.fetchDictionary)
        .onDisappear(perform: trackScreenDisappearing)
        .actionSheet(isPresented: $viewModel.isDirectionsActionSheetPresented) {
            ActionSheet(
                title: Text(viewModel.actionSheetTitle),
                buttons: viewModel.actionSheetButtons
            )
        }
    }
}

#if DEBUG
struct AppointmentDetailView_Previews: PreviewProvider {
    static var previews: some View {
        AppointmentDetailView(
            activity: loadPreviewCarePlans()
                .flatMap { $0.activities }
                .first!
        ).environment(\.colorScheme, .light)

        AppointmentDetailView(
            activity: loadPreviewCarePlans()
                .flatMap { $0.activities }
                .first!
        ).environment(\.colorScheme, .dark)
    }
}
#endif
