// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import MaternityData
import MapKit
import CMSData
import CMSServices

extension AppointmentDetailView {
    final class ViewModel: ObservableObject {
        @Published var isDirectionsActionSheetPresented: Bool = false
        @Published var dictionary = CMSDictionary.empty()

        let activity: Activity

        var isPlannedOrFinished: Bool {
            activity.status == .planned || activity.status == .finished
        }

        var badgeText: String? {
            activity.status == .finished ? R.string.localizable.appointment_details_finished_badge() : nil
        }

        var actionSheetTitle: String {
            R.string.localizable.appointment_details_directions_sheet(activity.location?.address.formatted ?? activity.kind)
        }

        var actionSheetButtons: [ActionSheet.Button] {
            var buttons: [ActionSheet.Button] = []

            if UIApplication.shared.isGoogleMapsInstalled {
                buttons.append(.default(Text(R.string.localizable.ios_directions_google_maps()), action: { [weak self] in
                    if let location = self?.activity.location {
                        UIApplication.shared.openGoogleMaps(address: location.address.formatted)
                    }
                }))
            }
            buttons.append(.default(Text(R.string.localizable.ios_directions_apple_maps()), action: { [weak self] in
                if let location = self?.activity.location {
                    UIApplication.shared.openAppleMaps(address: location.address.formatted)
                }
            }))
            buttons.append(.cancel(Text(R.string.localizable.common_regret())))

            return buttons
        }

        @DI private var dictionaryService: DictionaryService
        private var cancelBag = CancelBag()

        init(activity: Activity) {
            self.activity = activity
        }

        func fetchDictionary() {
            dictionaryService
                .fetchDictionaryWithEmptyFallback()
                .sink { (dictionary: CMSDictionary) in
                    self.dictionary = dictionary
                }
                .store(in: &cancelBag)
        }
    }
}
