// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import MessageUI

struct AppointmentDetailPlannedValues: View {
    let from: Date
    let to: Date
    let location: String
    let practitioner: String
    let phoneNumber: String?
    let email: String?
    
    @State private var presentSendEmail: Bool = false
    @State private var contactValue: String?
    @State private var presentCopySheet: Bool = false
    
    var body: some View {
        VStack(
            alignment: .leading,
            spacing: Metrics.Spacing.appointmentsDisplayValuesSpacing) {
            AppointmentDetailValueDisplay(
                title: R.string.localizable.appointment_details_date_label(),
                value: Text(AppointmentsView.dateFormatter.string(from: from)).applyTextStyle(fontStyle: .body)
            )
            AppointmentDetailValueDisplay(
                title: R.string.localizable.appointment_time_label(),
                value: Text("\(AppointmentsView.timeFormatter.string(from: from)) - \(AppointmentsView.timeFormatter.string(from: to))")
                    .applyTextStyle(fontStyle: .body)
            )
            AppointmentDetailValueDisplay(
                title: R.string.localizable.appointment_location_label(),
                value: Text(location).applyTextStyle(fontStyle: .body)
            )
            AppointmentDetailValueDisplay(
                title: R.string.localizable.appointment_at_label(),
                value: Text(practitioner).applyTextStyle(fontStyle: .body)
            )
            if phoneNumber != nil || email != nil {
                AppointmentDetailValueDisplay(
                    title: R.string.localizable.appointment_details_contact_label(),
                    value: VStack(alignment: .leading) {
                        if let phoneNumber = phoneNumber {
                            AppointmentDetailContactButton(
                                text: phoneNumber,
                                action: {
                                    if UIApplication.shared.canOpenPhoneUrl() {
                                        UIApplication.shared.callPhoneNumber(phoneNumber: phoneNumber)
                                    } else {
                                        contactValue = phoneNumber
                                        presentCopySheet = true
                                    }
                                },
                                image: R.image.phone
                            )
                        }
                        if let email = email {
                            AppointmentDetailContactButton(
                                text: email,
                                action: {
                                    contactValue = email
                                    if MFMailComposeViewController.canSendMail() {
                                        presentSendEmail = true
                                    } else {
                                        presentCopySheet = true
                                    }
                                },
                                image: R.image.phone
                            )
                        }
                    }
                )
            }
        }
        .alert(
            isPresented: $presentCopySheet,
            content: {
                Alert(
                    title: Text(R.string.localizable.ios_contact_method_copy_alert_title()),
                    message: Text(R.string.localizable.ios_contact_method_copy_alert_text(contactValue ?? "")),
                    primaryButton: .default(
                        Text(R.string.localizable.ios_contact_method_copy_alert_confirm()),
                        action: { UIPasteboard.general.string = contactValue }
                    ),
                    secondaryButton: .cancel()
                )
            }
        )
        .sheet(isPresented: $presentSendEmail, content: {
            MailModal(isShowing: $presentSendEmail, result: .constant(nil))
        })
    }
}

struct AppointmentDetailPlannedValues_Previews: PreviewProvider {
    static var previews: some View {
        AppointmentDetailPlannedValues(
            from: Date(),
            to: Date(),
            location: "Europaplads 2, 1., 8000 Aarhus C",
            practitioner: "Dr. Trifork",
            phoneNumber: "88 88 88 88",
            email: "info@trifork.com"
        )
    }
}
