// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import SwiftUI
import MaternityData
import MaternityServices

extension AppointmentsView {
    final class ViewModel: ObservableObject {

        @DI private var userContext: UserContext
        @DI private var carePlanService: CarePlanService

        @Published var state = DataLoadingState.initial
        @Published var finishedActivities = [CareplanActivityYear]()
        @Published var plannedActivities = [CareplanActivityYear]()

        private var cancelBag = CancelBag()

        func fetchData() {
            // This endpoint is really expensive, so we will only load data once!
            guard state.allowsDataFetch else { return }

            state = .loading
            carePlanService
                .fetchCarePlans()
                .compactMap { $0.first }
                .map { carePlan -> (finished: [CareplanActivityYear], planned: [CareplanActivityYear]) in
                    let finished = carePlan.activities.filter { $0.status == .finished }.mapToYearAppointmentModel()
                    let planned = carePlan.activities.filter { $0.status != .finished }.mapToYearAppointmentModel()

                    return (finished: finished, planned: planned)
                }
                .receiveOnMain()
                .sink { [weak self] completion in
                    switch completion {
                    case .failure(let error):
                        self?.state = .error(error)
                    default: break
                    }
                } receiveValue: { finished, planned in
                    self.finishedActivities = finished
                    self.plannedActivities = planned
                    self.state = .data
                }
                .store(in: &cancelBag)
        }
    }

    struct MonthAppointmentModel: Identifiable {
        let month: Int
        let activities: [Activity]

        var id: Int { month }
    }
}

fileprivate extension Array where Element == Activity {
    func mapToYearAppointmentModel() -> [CareplanActivityYear] {
        self
            .compactMap { $0.sortDate.year }
            .toSet()
            .sorted().map { year -> CareplanActivityYear in
                let yearlyActivities = self.filter { $0.sortDate.year == year }
                let months = yearlyActivities.compactMap { $0.sortDate.month }.toSet()

                let monthlyActivities = months
                    .sorted()
                    .map { month in CareplanActivitiesMonth(month: month, activities: yearlyActivities.filter { $0.sortDate.month == month }) }

                return CareplanActivityYear(year: year, months: monthlyActivities)
            }
    }
}
