// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import MaternityData

struct PreviousPregnanciesView: View, ScreenTrackable {
    enum Section {
        case births
        case abortions
    }

    @Environment(\.viewController) private var viewControllerHolder: UIViewController?
    @ObservedObject private var viewModel = ViewModel()
    @State private var selectedSegment = Section.births

    var body: some View {
        BottomContainerCardView {
            VStack(spacing: 0) {
                Picker(selection: $selectedSegment, label: Text("")) {
                    Text(R.string.localizable.journal_section_previous_births_tab_title()).tag(Section.births)
                    Text(R.string.localizable.journal_section_previous_abortions_tab_title()).tag(Section.abortions)
                }.pickerStyle(SegmentedPickerStyle())
                .padding(.horizontal)

                switch selectedSegment {
                case .births: birthsBody
                case .abortions: abortionsBody
                }
            }
        }
        .padding(.top)
        .onAppear(perform: viewModel.fetchData)
        .navigationBarTitle(Text(R.string.localizable.journal_section_around_the_pregnancy_past_pregnancies_item()), displayMode: .inline)
        .onAppear(perform: trackScreenAppearing)
        .onDisappear(perform: trackScreenDisappearing)
    }

    private var birthsBody: some View {
        PreviousPregnanciesTabView(state: viewModel.birthState, retryAction: viewModel.fetchData) {
            ForEach(viewModel.births) { birth in
                VStack(alignment: .leading) {
                    ForEach(birth.table.indices) { row in
                        Text(R.string.localizable.journal_section_previous_births_child_title(viewModel.births.count == 1 ? "" : "\(row + 1)"))
                            .applyTextStyle(fontStyle: .h4)

                        TwoColumnDataTable(
                            rows: birth.table[row].map(mapToCell)
                        )
                            .padding(.bottom)

                        ForEach(birth.info) { keyValueModel in
                            KeyValueView(title: keyValueModel.title,
                                         primaryValue: keyValueModel.primaryValue,
                                         secondaryValue: keyValueModel.secondaryValue,
                                         dictionaryAction: keyValueModel.dictionaryValue == nil ? nil : {
                                            showDictionaryView(title: keyValueModel.title, text: keyValueModel.dictionaryValue ?? R.string.localizable.common_missing_data())
                                         })
                                .padding(.top)
                        }
                    }
                }
                .padding(.bottom)
            }
        }
    }

    private var abortionsBody: some View {
        PreviousPregnanciesTabView(state: viewModel.abortionState, retryAction: viewModel.fetchData) {
            ForEach(viewModel.abortions) { (abortion: Abortion) in
                VStack(alignment: .leading) {
                    ForEach(abortion.table.indices) { row in
                        Text(
                            R.string.localizable.journal_section_previous_abortions_abortion_title(
                                abortion.table.count == 1 ? "" : "\(row + 1)"
                            )
                        )
                        .applyTextStyle(fontStyle: .h4)
                        
                        TwoColumnDataTable(
                            rows: abortion.table.map(mapToCell)
                        )
                        .padding(.bottom)
                    }
                }
                .padding(.bottom)
            }
        }
    }

    private func mapToCell(_ keyValueModel: KeyValueModel) -> [DataTableCell] {
        [
            DataTableCell(id: "title-\(keyValueModel.id)",
                          value: keyValueModel.title,
                          type: keyValueModel.dictionaryValue == nil ? .text : .button({
                            showDictionaryView(title: keyValueModel.title, text: keyValueModel.dictionaryValue ?? R.string.localizable.common_missing_data())
                          }),
                          fixedWidth: 150),

            DataTableCell(id: "value-\(keyValueModel.id)",
                          value: keyValueModel.primaryValue ?? R.string.localizable.common_missing_data(),
                          type: .text,
                          fixedWidth: nil)
        ]
    }

    private func showDictionaryView(title: String, text: String) {
        viewControllerHolder?.presentPartialSheet(
            buttonColor: R.color.infoButton.color(),
            content: {
                InfoSheetContent(
                    title: title,
                    text: text
                )
            })
    }
}

struct PreviousPregnanciesView_Previews: PreviewProvider {
    static var previews: some View {
        PreviousPregnanciesView()
    }
}
