// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import MaternityServices
import MaternityData
import Combine
import ErrorData

extension JournalMeasurementsView {
    class ViewModel: ObservableObject {
        @Published var chartValues: [MeasurementsLineChart.Value] = []
        @Published var chartArea = MeasurementsLineChart.Area(upper: [], lower: [], middle: [])
        @Published var chartXUnit = ""
        @Published var chartYUnit = ""
        @Published var state: DataLoadingState = .initial
        @Published var bodyMeasureValues: [KeyValueModel] = []

        @Published var titleColumn: [DataTableCell] = []
        @Published var valueColumns: [[DataTableCell]] = []

        @DI private var observationTableService: ObservationTableService
        @DI private var bodyMeasureService: BodyMeasureService
        
        private var cancelBag = CancelBag()

        func fetchData() {
            state = .loading
            bodyMeasureService.fetchBodyMeasureData()
                .receiveOnMain()
                .flatMap { [weak self] (bodyMeasureData: [KeyValueModel]) -> AnyPublisher<DataTableClinicalResult, ServiceError> in
                    self?.bodyMeasureValues = bodyMeasureData
                    return self?.observationTableService.fetchClinicalTable(titleWidth: 140, valueWidth: 100) ??
                        Just(DataTableClinicalResult(
                                tableColumnSet: DataTableColumnSet(
                                    titleColumn: [],
                                    valueColumns: []
                                ),
                                pubisMeasurements: []
                        ))
                        .setFailureType(to: ServiceError.self)
                        .eraseToAnyPublisher()
                }
                .receiveOnMain()
                .sink(
                    receiveCompletion: { [weak self] (completion) in
                        switch completion {
                        case .failure(let error):
                            self?.state = .error(error)
                        case .finished:
                            break
                        }
                    },
                    receiveValue: { [weak self] (result: DataTableClinicalResult) in
                        self?.state = .data
                        self?.chartValues = result.pubisMeasurements
                            .map({ MeasurementsLineChart.Value(
                                x: $0.maternityWeek,
                                y: $0.measurement.value
                            )})
                        self?.chartArea = MeasurementsLineChart.Area.mock()
                        self?.chartXUnit = "uger"
                        self?.chartYUnit = "cm"
                        self?.titleColumn = result.tableColumnSet.titleColumn
                        self?.valueColumns = result.tableColumnSet.valueColumns
                    })
                .store(in: &cancelBag)

        }
    }

    class PreviewViewModel: ViewModel {
        override func fetchData() {
            chartValues = MeasurementsLineChart.Value.mockList()
            chartArea = MeasurementsLineChart.Area.mock()
            chartXUnit = "uger"
            chartYUnit = "cm"
            state = .data
            bodyMeasureValues = [KeyValueModel(title: "Preview", primaryValue: "Test")]
            valueColumns = ScrollableDataTable.previewColumns()
            titleColumn = ScrollableDataTable.previewTitleColumn()
        }
    }
}
