// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import TIM
import Combine

extension OnboardingConsentView {
    final class ViewModel: ObservableObject {
        private var cancelBag = CancelBag()
        @DI private var logger: Logger
        @DI private var timAuth: TIMAuth
        @DI private var timStorage: TIMDataStorage

        @Published var oidcLoginFailed: Bool = false
        @Published var oidcLoginSucceded: Bool = false
        @Published var userId: String?

        func performOpenIDConnectLogin() {
            guard let rootViewController = App.windowRootViewController else {
                return
            }
            self.oidcLoginFailed = false
            self.oidcLoginSucceded = false

            timAuth.performOpenIDConnectLogin(presentingViewController: rootViewController)
                .sink { [weak self] (completion) in
                    switch completion {
                    case .failure(let error):
                        self?.logger.error("Failed to login with OIDC: \(error.localizedDescription)")
                        self?.oidcLoginFailed = true
                    case .finished:
                        guard let userId = self?.userId else {
                            self?.oidcLoginSucceded = false
                            self?.oidcLoginFailed = true
                            return
                        }

                        // New login succeeded. Since we only support one user login we will clear all other ids on this device.
                        self?.timStorage.clearAllUsers(exceptUserId: userId)

                        NotificationCenter.default.post(
                            notification: NSNotification.Name.SceneDelegate.changeRootViewController,
                            value: SceneDelegate.NotificationMessage.beginRegisterFlow(userId: userId)
                        )
                    }
                } receiveValue: { [weak self] (accessToken) in
                    self?.logger.debug("Received access token from OIDC login: \(accessToken)")
                    self?.userId = accessToken.userId
                }.store(in: &cancelBag)

        }
    }
}
