// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import SwiftUI
import Charts

final class MeasurementsLineChart : LineChartView {
    let config: Config
    let values: [Value]
    let area: Area

    required init(config: Config, values: [Value], area: Area) {
        self.config = config
        self.values = values
        self.area = area
        super.init(frame: .zero)
        setup()
    }

    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    private func setup() {
        renderer = MeasurementsLineChartRenderer(dataProvider: self, animator: chartAnimator, viewPortHandler: viewPortHandler)

        let transformer = getTransformer(forAxis: .left)
        setupXAxis(transformer)
        setupLeftAxis(transformer)

        noDataText = ""
        rightAxis.enabled = false
        legend.enabled = false

        setupDataSets()
    }

    private func setupXAxis(_ transformer: Transformer) {
        xAxis.valueFormatter = MeasurementLineChartXAxisLabelFormatter(unit: config.xUnit)
        xAxis.spaceMin = 1
        xAxis.spaceMax = 1
        xAxis.axisLineColor = R.color.measurementsChartLines()!
        xAxis.yOffset = 8
        xAxis.labelPosition = .bottom
        xAxis.labelFont = FontStyle.h5.scaledFont()
        xAxis.labelTextColor = R.color.text()!
        xAxis.labelCount = 7
        xAxis.drawGridLinesEnabled = false
        xAxis.drawAxisLineEnabled = true
        xAxisRenderer = MeasurementsLineChartXAxisRenderer(
            viewPortHandler: viewPortHandler,
            xAxis: xAxis,
            transformer: transformer
        )
    }

    private func setupLeftAxis(_ transformer: Transformer) {
        leftAxis.valueFormatter = MeasurementLineChartYAxisLabelFormatter(unit: config.yUnit)
        leftAxis.spaceMin = 1
        leftAxis.spaceMax = 1
        leftAxis.axisMinimum = 0
        leftAxis.drawAxisLineEnabled = false
        leftAxis.labelFont = FontStyle.h5.scaledFont()
        leftAxis.labelTextColor = R.color.text()!
        leftAxis.labelCount = 6
        leftAxis.drawGridLinesEnabled = true
        leftAxis.gridColor = R.color.measurementsChartLines()!
        leftYAxisRenderer = MeasurementsLineChartYAxisRenderer(
            viewPortHandler: viewPortHandler,
            yAxis: leftAxis,
            transformer: transformer
        )
    }

    private func setupDataSets() {
        let valuesDataSet = MeasurementLineChartValueDataSet(
            entries: values.toChartEntries()
        )

        let areaDataSet = MeasurementLineChartAreaDataSet(
            lower: area.lower.toChartEntries(),
            upper: area.upper.toChartEntries(),
            middle: area.middle.toChartEntries()
        )

        data = LineChartData(
            dataSets: [
                areaDataSet,
                valuesDataSet
            ]
        )
    }

    struct Config {
        let xUnit: String
        let yUnit: String
    }

    struct Value {
        let x: Double
        let y: Double

        static func mockList() -> [Value] {
            stride(from: 0, to: 40, by: 10)
                .map({ Value(x: $0, y: Double.random(in: 15...40))})
        }
    }

    struct Area {
        let upper: [Value]
        let lower: [Value]
        let middle: [Value]

        private static func strideForValues(min: Double, max: Double) -> [MeasurementsLineChart.Value] {
            stride(from: 0, to: 42, by: 0.6)
                .map({ MeasurementsLineChart.Value(x: $0, y: Double.random(in: min...max)) })
        }
        static func mock() -> Area {
            Area(
                upper: strideForValues(min: 30, max: 40),
                lower: strideForValues(min: 0, max: 15),
                middle: strideForValues(min: 20, max: 30)
            )
        }
    }
}

struct SUIMeasurementsLineChart: UIViewRepresentable {
    let config: MeasurementsLineChart.Config
    let values: [MeasurementsLineChart.Value]
    let area: MeasurementsLineChart.Area

    func makeUIView(context: Context) -> MeasurementsLineChart {
        MeasurementsLineChart(config: config, values: values, area: area)
    }

    func updateUIView(_ uiView: MeasurementsLineChart, context: Context) {

    }
}

fileprivate extension Sequence where Element == MeasurementsLineChart.Value {
    func toChartEntries() -> [ChartDataEntry] {
        self.map({ ChartDataEntry(x: $0.x, y: $0.y) })
    }
}

struct SUIMeasurementsLineChart_Previews: PreviewProvider {
    static var previews: some View {
        SUIMeasurementsLineChart(
            config: MeasurementsLineChart.Config(
                xUnit: "uger",
                yUnit: "cm"
            ),
            values: MeasurementsLineChart.Value.mockList(),
            area: MeasurementsLineChart.Area.mock()
        )
    }
}
