// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import MaternityData

struct DataTableCellView: View {
    let cell: DataTableCell
    let rowNumber: Int
    let height: CGFloat

    var body: some View {
        VStack(alignment: .leading, spacing: 0) {
            Spacer()
            HStack(spacing: 0) {
                switch cell.type {
                case .text:
                    Text(cell.value)
                        .modifier(
                            DataTableCellViewModifier(
                                fontStyle: .body,
                                foregroundColor: R.color.text.color(),
                                rowNumber: rowNumber
                            )
                        )
                    
                case .boldText:
                    Text(cell.value)
                        .modifier(
                            DataTableCellViewModifier(
                                fontStyle: .h4,
                                foregroundColor: R.color.text.color(),
                                rowNumber: rowNumber
                            )
                        )
                    
                case .button(let action):
                    Button(cell.value, action: action)
                        .modifier(
                            DataTableCellViewModifier(
                                fontStyle: .h4,
                                foregroundColor: R.color.infoButton.color(),
                                rowNumber: rowNumber
                            )
                        )

                }
                Spacer(minLength: 0)
            }

            Spacer()
        }
        .frame(height: height)
        .frame(minHeight: 64)
        .modifier(OptionalWidthViewModifier(width: cell.fixedWidth))
        .background(rowNumber % 2 != 0 ? R.color.tableRow.color() : Color.clear)
    }
}

private struct DataTableCellViewModifier: ViewModifier {
    let fontStyle: FontStyle
    let foregroundColor: Color
    let rowNumber: Int

    func body(content: Content) -> some View {
        content
            .fixedSize(horizontal: false, vertical: true) // To make iOS 13 happy
            .font(Font(fontStyle.scaledFont()))
            .foregroundColor(foregroundColor)
            .padding(10)
            .overlay(DataTableCellDetermineHeight(row: rowNumber))
    }
}

struct DataTableCellView_Previews: PreviewProvider {
    static var previews: some View {
        ScrollView {
            VStack(spacing: 0) {
                DataTableCellView(
                    cell: DataTableCell(
                        id: "id",
                        value: String.loremIpsum(.small),
                        type: .boldText,
                        fixedWidth: 100
                    ),
                    rowNumber: 0,
                    height: 400
                )
                DataTableCellView(
                    cell: DataTableCell(
                        id: "id",
                        value: "YAY",
                        type: .button({ }),
                        fixedWidth: 100
                    ),
                    rowNumber: 1,
                    height: 20
                )
                DataTableCellView(
                    cell: DataTableCell(
                        id: "id",
                        value: "YAY",
                        type: .text,
                        fixedWidth: 100
                    ),
                    rowNumber: 2,
                    height: 70
                )
            }
            .border(Color.pink, width: 2)
        }
    }
}
