// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import MaternityData

struct ScrollableDataTable: View {
    let valueColumns: [[DataTableCell]]
    let titleColumnRows: [DataTableCell]

    @Binding var scrollOffset: CGPoint
    @Binding var scrollContentSize: CGSize
    @Binding var scrollViewSize: CGSize

    @State private var calculatedHeightForRow: [Int: CGFloat] = [:]

    var body: some View {
        HStack(alignment: .top, spacing: 0) {
            buildColumn(titleColumnRows)
            ScrollViewOffset(
                scrollOffset: $scrollOffset,
                scrollContentSize: $scrollContentSize,
                axes: .horizontal,
                showsIndicators: false) {
                HStack(spacing: 0) {
                    ForEach(valueColumns.indices, id: \.self) { columnNumber in
                        let rowValuesForColumn = valueColumns[columnNumber]
                        buildColumn(rowValuesForColumn)
                            .tag(columnNumber)
                        DefaultDivider()
                    }
                }
            }
            .overlay(
                HStack {
                    LinearGradient(
                        gradient: Gradient(colors: [R.color.shadow.color(), R.color.shadow.color().opacity(0)]), startPoint: .leading, endPoint: /*@START_MENU_TOKEN@*/.trailing/*@END_MENU_TOKEN@*/
                    )
                    .frame(width: 10)
                    Spacer()
                }
            )
            .onSizeChanged(ScrollViewSizeReaderKey.self) { (size) in
                scrollViewSize = size
            }

        }

    }

    private func buildColumn(_ column: [DataTableCell]) -> some View {
        VStack(alignment: .leading, spacing: 0) {
            ForEach(column.indices, id: \.self) { rowNumber in
                let cell = column[rowNumber]
                DataTableCellView(
                    cell: cell,
                    rowNumber: rowNumber,
                    height: calculatedHeightForRow[rowNumber] ?? 0
                )
                .onPreferenceChange(DataTableCellDetermineHeight.Key.self) { result in
                    calculatedHeightForRow[result.row] = max(
                        calculatedHeightForRow[result.row] ?? 0,
                        result.height
                    )
                }
                .tag(cell.id)
            }
        }
    }

    private static var previewRows: Int = 5

    static func previewTitleColumn() -> [DataTableCell] {
        (1...previewRows).map({ (row: Int) -> DataTableCell in
            DataTableCell(
                id: "1-\(row)",
                value: (1...Int.random(in: 1...7)).map({ _ in "Test"}).joined(separator: " ") + "-1-\(row)",
                type: .boldText,
                fixedWidth: 140
            )
        })
    }

    static func previewColumns() -> [[DataTableCell]] {
        (2...7).map({ (column: Int) -> [DataTableCell] in
            (1...previewRows).map({ (row: Int) -> DataTableCell in
                DataTableCell(
                    id: "\(column)-\(row)",
                    value: (1...Int.random(in: 1...2)).map({ _ in "Test"}).joined(separator: " ") + "-\(column)-\(row)",
                    type: .text,
                    fixedWidth: 100
                )
            })
        })
    }
}

private struct ScrollViewSizeReaderKey: SizeReaderKey {
    static var defaultValue: CGSize = .zero
}

struct ScrollableDataTable_Previews: PreviewProvider {
    static var previews: some View {
        ScrollView {
            ScrollableDataTable(
                valueColumns: ScrollableDataTable.previewColumns(),
                titleColumnRows: ScrollableDataTable.previewTitleColumn(),
                scrollOffset: .constant(.zero),
                scrollContentSize: .constant(.zero),
                scrollViewSize: .constant(.zero)
            )
        }
    }
}
