// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import ErrorData

enum DataLoadingState {
    case initial
    /// Title, Description
    case empty(String, String)
    case data
    case loading
    case error(Error)

    var allowsDataFetch: Bool {
        let allow: Bool

        switch self {
        case .initial,
             .error,
             .empty:
            allow = true
        case .loading, .data:
            allow = false
        }

        return allow
    }
}

extension DataLoadingState: Equatable {
    static func == (lhs: DataLoadingState, rhs: DataLoadingState) -> Bool {
        switch (lhs, rhs) {
        case (.initial, .initial),
             (.data, .data),
             (.loading, .loading):
                return true

        case (.empty(let lhsTitle, let lhsDescription), .empty(let rhsTitle, let rhsDescription)):
            return lhsTitle == rhsTitle && lhsDescription == rhsDescription

        case (.error(let lhsError), .error(let rhsError)):
            return lhsError.localizedDescription == rhsError.localizedDescription
        default:
            return false
        }
    }
}

struct DataLoadingView<Content: View>: View {

    let state: DataLoadingState
    let retryAction: () -> Void
    let content: Content
    let animateStateChanges: Bool

    init(state: DataLoadingState, animateStateChanges: Bool = true, retryAction: @escaping () -> Void, @ViewBuilder content: @escaping () -> Content) {
        self.state = state
        self.animateStateChanges = animateStateChanges
        self.retryAction = retryAction
        self.content = content()
    }

    var body: some View {
        Group {
            switch state {
            case .empty(let title, let description):
                VStack(spacing: 0) {
                    Spacer()
                    EmptyStateView(title: title, text: description)
                    Spacer()
                }
            case .data:
                self.content
            case .initial,
                 .loading:
                VStack {
                    Spacer()
                    ActivityIndicatorView(isAnimating: true,
                                          style: .medium,
                                          hidesWhenStopped: true)
                    Spacer()
                }

            case .error(let error):
                VStack(spacing: 0) {
                    Spacer()
                    ErrorStateView(
                        text: error.localizedDescription,
                        retryAction: retryAction
                    )
                    Spacer()
                }
            }
        }
        .transition(
            animateStateChanges ? AnyTransition.opacity.animation(.easeInOut(duration: 0.2)) : .identity
        )
    }
}

struct DataLoadingView_Previews: PreviewProvider {
    static var previews: some View {
        VStack {
            DataLoadingView(state: .empty("Empty!", "Øv, der kom ingen data!"), retryAction: {}, content: {
                Text("Should not be shown!")
            })
            DataLoadingView(state: .data, retryAction: {}, content: {
                Text("Content! 🥳")
            })
            DataLoadingView(state: .loading, retryAction: {}, content: {
                Text("Should not be shown!")
            })
            DataLoadingView(state: .error(ServiceError.failedToParseResponse), retryAction: {}, content: {
                Text("Should not be shown!")
            })
        }
    }
}
