// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct PartialSheetView<PresentedView: View>: View {
    @Environment(\.viewController) private var viewControllerHolder: UIViewController?

    var buttonColor = R.color.primary.color()
    let presentedView: () -> PresentedView

    // We need to use this because an bool is just flipped and has no progress.
    // Thus the view controller dismiss animation will be called right away.
    @State private var presentingAnimationProgress: CGFloat = 0.0
    private var isPresented: Bool { presentingAnimationProgress >= 1.0 }
    private let margin: CGFloat = 12

    var body: some View {
        GeometryReader { reader in
            ZStack {
                VisualBlurEffectView(style: .systemChromeMaterialDark)
                    .edgesIgnoringSafeArea(.all)
                    .opacity(self.isPresented ? 0.9 : 0)
                    .onTapGesture(perform: closeSheet)

                VStack(spacing: 0) {
                    Spacer()

                    self.presentedView()
                        .padding()
                        .background(R.color.cardBackground.color())
                        .cornerRadius(Metrics.Sizes.cardCornerRadius)
                        .frame(width: reader.size.width - margin * 2)

                    self.toggleButton
                        .frame(width: reader.size.width - margin * 2)
                        .padding(.vertical)
                }
                .background(Color.clear)
                .offset(x: 0, y: self.isPresented ? 0 : reader.frame(in: .global).height)
            }
        }
        .onAppear {
            withAnimation(.spring()) {
                self.presentingAnimationProgress = 1
            }
        }
    }

    private var toggleButton: some View {
        Button(action: closeSheet, label: {
            HStack {
                Spacer()
                Text(R.string.localizable.common_cta_close())
                    .applyTextStyle(fontStyle: .h4, textColor: .white)
                    .padding()

                Spacer()
            }
            .background(buttonColor)
            .cornerRadius(8)
        })
        .onAnimationCompleted(for: presentingAnimationProgress) {
            guard !self.isPresented else { return } // To avoid it dismisses when
            viewControllerHolder?.dismiss(animated: false, completion: nil)
        }
    }

    private func closeSheet() {
        withAnimation {
            self.presentingAnimationProgress = 0
        }
    }
}

struct PartialSheetView_Previews: PreviewProvider {
    static var previews: some View {
        PartialSheetView {
            InfoSheetContent(title: "Text", text: String.loremIpsum(.small))
        }
    }
}
