// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import Combine
import TIM

protocol ActivityMonitor {
    var shouldRequireLoginOnAppActive: Bool { get }
    func setupForRootViewControllerChanges()
    func startTracking()
    func stopTracking()
    func appBecameInactive()
}

final class ActivityMonitorWindow: UIWindow, ActivityMonitor {

    @DI private var logger: Logger
    @DI private var userContext: UserContextProtocol

    private var cancelBag = CancelBag()
    private let allowedTimeInBackground: TimeInterval = 60 * 5
    private let allowedInactivityTimeOnScreen: TimeInterval = 60 * 15
    private (set) var inactivityTimer: Timer?

    private var appBecameInactiveDate: Date?

    private var lastActivityRegistrationDate: Date = Date()

    init() {
        super.init(frame: UIScreen.main.bounds)
        setup()
    }

    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        setup()
    }

    private func setup() {
        rootViewController = UIViewController()
        rootViewController?.view = UIView(frame: bounds)
        rootViewController?.view.autoresizingMask = [ .flexibleWidth, .flexibleHeight ]
        rootViewController?.view.backgroundColor = .clear
        rootViewController?.view.alpha = 1
    }

    func setupForRootViewControllerChanges() {
        let keyWindow = UIApplication.shared.windows.first(where: { $0.isKeyWindow })
        keyWindow?.addSubview(self)
        windowLevel = UIWindow.Level.normal + 1
        isHidden = false
    }

    func startTracking() {
        updateInactivityTimer(lastActivityDate: Date())
    }

    func stopTracking() {
        cancelInactivityTimer()
    }

    func appBecameInactive() {
        appBecameInactiveDate = Date()
    }

    override func point(inside point: CGPoint, with event: UIEvent?) -> Bool {
        let now: Date = Date()

        // Let's only reset the user inactivity timer every 10 seconds.
        if lastActivityRegistrationDate.timeIntervalSince(now) < -10 && inactivityTimer != nil {
            lastActivityRegistrationDate = now
            updateInactivityTimer(lastActivityDate: now)
            logger.debug("User inactivity timer was reset.")
        }

        return false
    }

    // MARK: - Inactivity
    private func updateInactivityTimer(lastActivityDate: Date) {
        cancelInactivityTimer()
        logger.debug("Inactivity timer was started!")

        let timeoutDate: Date = lastActivityDate.addingTimeInterval(allowedInactivityTimeOnScreen)
        let timer: Timer = Timer(fire: timeoutDate, interval: 0, repeats: false, block: { timer in
            guard timer == self.inactivityTimer else { return }
            guard self.userContext.isAuthenticated else { return }
            self.logger.info("Inactivity timeout! Last activity was registered at: \(lastActivityDate)")
            self.cancelInactivityTimer()

            DispatchQueue.main.async {
                if let userId = self.userContext.userId {
                    NotificationCenter.default.post(
                        notification: NSNotification.Name.SceneDelegate.changeRootViewController,
                        value: SceneDelegate.NotificationMessage.showLoginWithInactivityAlert(userId: userId)
                    )
                } else {
                    self.logger.error("Resetting to onboarding due to inactivity. This should have been the login screen - something went wrong, while getting the current user identifier.")
                    NotificationCenter.default.post(
                        notification: NSNotification.Name.SceneDelegate.changeRootViewController,
                        value: SceneDelegate.NotificationMessage.resetToOnboarding
                    )
                }
            }
        })
        inactivityTimer = timer
        RunLoop.main.add(timer, forMode: RunLoop.Mode.common)
    }

    private func cancelInactivityTimer() {
        inactivityTimer?.invalidate()
        inactivityTimer = nil
        logger.debug("Inactivity timer was cancelled.")
    }

    // MARK: - Background handling
    var shouldRequireLoginOnAppActive: Bool {
        let shouldRequire: Bool
        if let inactiveDate: Date = appBecameInactiveDate {
            shouldRequire = inactiveDate.timeIntervalSinceNow <= -allowedTimeInBackground // Five minutes
        } else {
            shouldRequire = false
        }
        return shouldRequire
    }
}

final class DebugActivityMonitorWindow : ActivityMonitor {
    @DI var logger: Logger

    var shouldRequireLoginOnAppActive: Bool = false

    init() {

    }

    func setupForRootViewControllerChanges() {
        logger.debug("DEBUG-FAKE: Setting activity view up for root view controller changes.")
    }

    func startTracking() {
        logger.debug("DEBUG-FAKE: Activity tracking was started.")
    }

    func stopTracking() {
        logger.debug("DEBUG-FAKE: Activity tracking was stopped.")
    }

    func appBecameInactive() {
        logger.debug("DEBUG-FAKE: App became inactive.")
    }
}
