// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import SwiftUI

protocol SUIViewController: UIViewController {
    var hostingController: UIHostingController<AnyView> { get }
    var uiContainerView: UIView { get }
    func assignContentToRootView<Content: View>(_ content: Content)
}

extension SUIViewController {
    func assignContentToRootView<Content: View>(_ content: Content) {
        hostingController.rootView = AnyView(content)
    }

    func setupCustomContainerView() {
        view.addSubview(uiContainerView, withEdgeInsets: .zero)
        hostingController.willMove(toParent: self)
        uiContainerView.addSubview(hostingController.view, withEdgeInsets: .zero)
        hostingController.didMove(toParent: self)
        hostingController.view.backgroundColor = .clear
    }
}

class SUIViewContainerViewController<T: UIView>: UIViewController, SUIViewController {
    var uiContainerView: UIView {
        specificTypeView
    }
    let specificTypeView: T
    var hostingController: UIHostingController<AnyView> = UIHostingController(rootView: AnyView(EmptyView()))

    required init(uiContainerView: T) {
        self.specificTypeView = uiContainerView
        super.init(nibName: nil, bundle: nil)
    }

    override init(nibName nibNameOrNil: String?, bundle nibBundleOrNil: Bundle?) {
        self.specificTypeView = T() // Create view if none was specificially given
        super.init(nibName: nibNameOrNil, bundle: nibBundleOrNil)
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        setupCustomContainerView()
    }
}

/// Wraps SwiftUI content in a specified view controller
struct SUIViewControllerWrapper<ViewController: SUIViewController, Content: View>:
    UIViewControllerRepresentable {

    var content: Content
    private var setupViewController: ((ViewController) -> Void)?

    init(@ViewBuilder content: @escaping () -> Content, setupViewController: ((ViewController) -> Void)? = nil) {
        self.content = content()
        self.setupViewController = setupViewController
    }

    func makeUIViewController(context: Context) -> ViewController {
        let viewController = ViewController()
        setupViewController?(viewController)
        viewController.assignContentToRootView(content)
        return viewController
    }

    func updateUIViewController(_ uiViewController: ViewController, context: Context) {
        uiViewController.assignContentToRootView(content)
    }
}
