// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct ScrollViewOffset<Content: View>: View {

    private let frameLayer = "frameLayer"
    let content: Content
    let axes: Axis.Set
    let showsIndicators: Bool

    @Binding var scrollOffset: CGPoint
    @Binding var contentSize: CGSize

    // Required to report the correct offset on iOS 13 🤦🏻‍♂️
    @State private var scrollViewPosition: CGPoint = .zero

    @DI private var logger: Logger

    init(
        scrollOffset: Binding<CGPoint>,
        scrollContentSize: Binding<CGSize> = .constant(.zero),
        axes: Axis.Set = .vertical,
        showsIndicators: Bool = true,
        @ViewBuilder content: @escaping () -> Content) {
        self._scrollOffset = scrollOffset
        self._contentSize = scrollContentSize
        self.content = content()
        self.axes = axes
        self.showsIndicators = showsIndicators
    }

    var body: some View {
        ScrollView(axes, showsIndicators: showsIndicators) {
            content
                .background(offsetReader)
        }
        .background(positionReader)
        .coordinateSpace(name: frameLayer)
        .onPreferenceChange(PositionPreferenceKey.self, perform: { position in
            scrollViewPosition = position
            logger.verbose("Scroll view global position: \(position)")
        })
        .onPreferenceChange(OffsetPreferenceKey.self, perform: { rect in
            let adjustOffset: CGPoint
            if #available(iOS 14, *) {
                adjustOffset = .zero
            } else {
                adjustOffset = scrollViewPosition
            }
            scrollOffset = CGPoint(x: rect.minX - adjustOffset.x, y: rect.minY - adjustOffset.y)
            contentSize = rect.size
            logger.verbose("Scrolling \(rect.origin) being adjust to: \(scrollOffset)")
        })
    }

    var offsetReader: some View {
        GeometryReader { proxy in
            Color.clear
                .preference(
                    key: OffsetPreferenceKey.self,
                    value: proxy.frame(in: .named(frameLayer))
                )
        }
    }

    var positionReader: some View {
        GeometryReader { proxy in
            Color.clear
                .preference(
                    key: PositionPreferenceKey.self,
                    value: proxy.frame(in: .global).origin
                )
        }
    }
}

private struct OffsetPreferenceKey: PreferenceKey {
    static var defaultValue: CGRect = .zero
  static func reduce(value: inout CGRect, nextValue: () -> CGRect) {}
}

private struct PositionPreferenceKey: PreferenceKey {
    static var defaultValue: CGPoint = .zero
    static func reduce(value: inout CGPoint, nextValue: () -> CGPoint) {}
}

struct ScrollViewOffset_Previews: PreviewProvider {
    static var previews: some View {
        ScrollViewOffset(
            scrollOffset: .constant(.zero),
            content: {
            Text("Content...")
        })
    }
}
