// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import XCTest
import MaternityData
@testable import MinGraviditet

final class ObservationTableTests: XCTestCase {

    func testConversionToDataTableCells() throws {
        let act1Date = Date().date(byAdding: .day, value: -5)
        let act1Id = "activity-1"

        let act2Date = Date().date(byAdding: .day, value: -1)
        let act2Id = "activity-2"

        let mother = ObservationTableScanMother(
            rows: [
                ObservationTableScanMotherRow(activityId: act1Id, date: act1Date, gestationAge: nil, amnioticFluid: nil, cervix: nil, dvp: nil, conclusion: nil),
                ObservationTableScanMotherRow(activityId: act2Id, date: act2Date, gestationAge: "22", amnioticFluid: nil, cervix: nil, dvp: nil, conclusion: nil)
        ])

        let fetuses = [
            ObservationTableScanFetus(
                id: "fetus-1",
                name: "Fetus 1",
                rows: [
                    ObservationTableScanFetusRow(
                        activityId: act1Id,
                        date: act1Date,
                        gestationAge: "23",
                        ulWeight: nil,
                        weightDeviation: nil,
                        fetusPresentation: "H"
                    )
                ]
            ),
            ObservationTableScanFetus(
                id: "fetus-2",
                name: "Fetus 2",
                rows: [
                    ObservationTableScanFetusRow(
                        activityId: act1Id,
                        date: act1Date,
                        gestationAge: "23",
                        ulWeight: nil,
                        weightDeviation: nil,
                        fetusPresentation: "H"
                    )
                ]
            )
        ]

        let scanTable = ObservationTableScan(mother: mother, fetuses: fetuses)
        let dataSet = scanTable.toDataTableColumnSet(dictionary: .empty(), titleWidth: 100, valueWidth: 100)

        // All columns should have the same number of cells, even though the fetuses has less activities than the mother
        XCTAssertEqual(dataSet.titleColumn.count, dataSet.valueColumns.first?.count)
        XCTAssertEqual(dataSet.titleColumn.count, dataSet.valueColumns.last?.count)
        XCTAssertEqual(dataSet.titleColumn.count, dataSet.valueColumns.last?.count)


        // Check that all titles are present in title column
        let allTitles = [
            "Fetus 1",
            "Fetus 2",
            R.string.localizable.appointment_details_measurement_mother(),
        ] + JournalTableScanFetusMeasurementType.allCases.map({ $0.title }) + JournalTableScanMotherMeasurementType.allCases.map({ $0.title })


        allTitles.forEach { (title) in
            XCTAssertTrue(dataSet.titleColumn.contains(where: { $0.value == title } ), "Missing title: \(title)")
            dataSet.valueColumns.forEach { (valueColumn) in
                XCTAssertFalse(valueColumn.contains(where: { $0.value == title })) // We shouldn't have any titles in the value columns
            }
        }

        XCTAssertTrue(dataSet.valueColumns.first!.contains(where: { $0.value == "22" })) // 22 should be in the first column for the mother
        XCTAssertFalse(dataSet.valueColumns.first!.contains(where: { $0.value == "H" })) // H should not be in the first column, since the first column should be act2
        XCTAssertTrue(dataSet.valueColumns.last!.contains(where: { $0.value == "H" })) // H should be in the second column
        XCTAssertTrue(dataSet.valueColumns.last!.contains(where: { $0.value == "23" })) // 23 should be in the second column
        XCTAssertTrue(dataSet.valueColumns.first!.contains(where: { $0.value == "-" })) // nil values should contain dashes.
        XCTAssertFalse(dataSet.valueColumns.first!.contains(where: { $0.value == "" })) // We shouldn't have any empty cells
    }
}
