// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import TIM
import DependencyInjection
import Combine
import XCTest

final class TestMockTIMAuth: TIMAuth {
    @Inject var timStorage: TIMDataStorage

    private var store = Set<AnyCancellable>()
    var isLoggedIn: Bool {
        refreshToken != nil && refreshToken?.userId != nil
    }
    
    var refreshToken: JWT?
    
    func logout() {
        refreshToken = nil
    }
    
    func handleRedirect(url: URL) {
        // Not relevant for test
    }
    
    func accessToken(_ completion: @escaping AccessTokenCallback) {
        notImplemented()
    }
    
    func performOpenIDConnectLogin(presentingViewController: UIViewController, completion: @escaping AccessTokenCallback) {
        notImplemented()
    }
    
    func loginWithPassword(userId: String, password: String, storeNewRefreshToken: Bool, completion: @escaping AccessTokenCallback) {
        notImplemented()
    }
    
    func loginWithBiometricId(userId: String, storeNewRefreshToken: Bool, completion: @escaping AccessTokenCallback) {
        notImplemented()
    }
    
    func accessToken() -> Future<JWT, TIMError> {
        Future { promise in
            promise(.success(mockAccessToken))
        }
    }
    
    func performOpenIDConnectLogin(presentingViewController: UIViewController) -> Future<JWT, TIMError> {
        Future { promise in
            let rt = mockRefreshToken
            self.refreshToken = rt
            promise(.success(rt))
        }
    }
    
    func loginWithPassword(userId: String, password: String, storeNewRefreshToken: Bool) -> Future<JWT, TIMError> {
        Future { promise  in
            self.timStorage.getStoredRefreshToken(userId: userId, password: password)
                .sink(receiveCompletion: { com in
                    switch com {
                    case .failure(let error):
                        promise(.failure(error))
                    case .finished:
                        break
                    }
                }, receiveValue: { (rt) in
                    self.refreshToken = rt
                    promise(.success(rt))
                })
                .store(in: &self.store)
        }
    }
    
    func loginWithBiometricId(userId: String, storeNewRefreshToken: Bool) -> Future<JWT, TIMError> {
        Future { promise in
            self.notImplemented()
            promise(.failure(TIMError.auth(.failedToGetRefreshToken)))
        }
    }
    
    private func notImplemented() {
        XCTFail("Function is not implemented for test.")
    }
}
