// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import HTTPNetworking
import ErrorData
import DependencyInjection
import Logger

public final class ResponseParser {
    @Inject private static var logger: LoggerProtocol

    public static func parseAndHandle<OutputType: Decodable>(input: Data, urlResponse: HTTPURLResponse?) throws -> OutputType {
        let result: OutputType
        let jsonDecoder = JSONDecoder()
        jsonDecoder.dateDecodingStrategy = .formatted(.customISO8601)

        switch urlResponse?.statusCode {
        case .some(200..<400):
            do {
                result = try jsonDecoder.decode(OutputType.self, from: input)
            } catch let e {
                logger.debug("Failed to parse result: \(e)")
                throw ServiceError.failedToParseResponse
            }
        case .some(400..<600):
            do {
                let error = try jsonDecoder.decode(ProblemDetails.self, from: input)
                throw ServiceError.backendError(error)
            } catch let e {
                logger.warning("Failed to parse result: \(e)")
                throw ServiceError.failedToParseResponse
            }
        default:
            throw ServiceError.unexpectedError(nil)
        }

        return result
    }
}
